#include "F_FrameRange.h"
#include <stdlib.h>
#include <string.h>

void FrameRange_callback(FrameRange* self, struct FrameContext* fc) {
	fc->startFrame = self->startFrame;
	fc->endFrame = self->endFrame;
}

FrameCallback* FrameRange_setup(FrameRange obj) {
	FrameCallback* fca = malloc(sizeof(struct FrameCallback));

	fca->callback = (FrameCallback_callback)FrameRange_callback;
	fca->callback_self = malloc(sizeof(obj));
	memcpy(fca->callback_self, &obj, sizeof(obj));

	return fca;
}
