#ifndef _FRAME_CALLBACK
#define _FRAME_CALLBACK

#include "../global.h"
#include "../RGBImage/RGBImage.h"

struct FrameContext {
	u32 frame;
	u32 startFrame;
	u32 endFrame;

	RGBImage frameBuffer;
};

typedef void (*FrameCallback_callback)(void* self, struct FrameContext* fc);

typedef struct FrameCallback {
	FrameCallback_callback callback;
	void* callback_self;
} FrameCallback;

#endif /* _FRAME_CALLBACK */
